# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('groups', '0002_discussion'),
    ]

    operations = [
        migrations.CreateModel(
            name='Comment',
            fields=[
                ('id', models.AutoField(primary_key=True, auto_created=True, verbose_name='ID', serialize=False)),
                ('body', models.TextField(max_length=255)),
                ('date_created', models.DateTimeField()),
                ('discussion', models.ForeignKey(to='groups.Discussion')),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
    ]



from django.conf.urls import include, url

from .views import comments, discussions, groups, subscriptions


urlpatterns = [
    url(r'^$', groups.GroupList.as_view(), name='group-list'),
    url(r'^(?P<pk>\d+)/', include([
        url(
            r'^$',
            groups.GroupDetail.as_view(),
            name='group-detail',
        ),
        url(
            r'^new-discussion/$',
            discussions.DiscussionCreate.as_view(),
            name='discussion-create',
        ),
        url(
            r'^subscribe/$',
            subscriptions.GroupSubscribe.as_view(),
            name='group-subscribe',
        ),
    ])),
    url(r'^discussions/(?P<pk>\d+)/', include([
        url(
            r'^$',
            discussions.DiscussionThread.as_view(),
            name='discussion-thread',
        ),
        url(
            r'^upload-file/$',
            comments.CommentUploadFile.as_view(),
            name='comment-upload-file',
        ),
        url(
            r'^subscribe/$',
            subscriptions.DiscussionSubscribe.as_view(),
            name='discussion-subscribe',
        ),
    ])),
    url(r'^comments/(?P<pk>\d+)/', include([
        url(
            r'^delete/$',
            comments.CommentDelete.as_view(),
            name='comment-delete',
        ),
    ])),
    url(
        r'^reply/',
        comments.CommentPostByEmail.as_view(),
        name='comment-reply',
    ),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('groups', '0013_update_filecomment_upload_update_basecomment_related_name'),
    ]

    operations = [
        migrations.AddField(
            model_name='discussion',
            name='ignorers',
            field=models.ManyToManyField(related_name='ignored_discussions', to=settings.AUTH_USER_MODEL),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('groups', '0003_comment'),
    ]

    operations = [
        migrations.AlterField(
            model_name='comment',
            name='body',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='comment',
            name='date_created',
            field=models.DateTimeField(auto_now_add=True),
        ),
        migrations.AlterField(
            model_name='discussion',
            name='date_created',
            field=models.DateTimeField(auto_now_add=True),
        ),
    ]



from django.views.generic import FormView
from django.views.generic.detail import SingleObjectMixin

from .. import forms, models


class SubscribeBase(SingleObjectMixin, FormView):
    form_class = forms.SubscribeForm

    def dispatch(self, request, *args, **kwargs):
        self.object = self.get_object()
        return super(SubscribeBase, self).dispatch(request, *args, **kwargs)

    def get_form_kwargs(self):
        """Pass the user to the form to check subscription state."""
        kwargs = super(SubscribeBase, self).get_form_kwargs()
        kwargs.update({
            'user': self.request.user,
            'instance': self.object,
            'url_name': self.subscribe_url_name,
        })
        return kwargs

    def form_valid(self, form):
        """Subscribe or unsubscribe the request user."""
        user = self.request.user

        if form.cleaned_data['subscribe']:
            self.object.subscribe(user)
        else:
            self.object.unsubscribe(user)

        return super(SubscribeBase, self).form_valid(form)

    def get_success_url(self):
        return self.object.get_absolute_url()


class DiscussionSubscribe(SubscribeBase):
    """Provide an endpoint for the subscribe/unsubscribe button."""
    model = models.Discussion
    template_name = 'groups/subscribe_button.html'
    subscribe_url_name = 'discussion-subscribe'


class GroupSubscribe(SubscribeBase):
    model = models.Group
    template_name = 'groups/group_subscribe_button.html'
    subscribe_url_name = 'group-subscribe'



import re

from django.apps import apps
from django.contrib import messages
from django.contrib.auth import get_user_model
from django.contrib.sites.shortcuts import get_current_site
from django.core import signing
from django.http import Http404, HttpResponse, HttpResponseRedirect
from django.shortcuts import get_object_or_404
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import View

from django.views.generic.edit import DeleteView

from ._helpers import CommentEmailMixin, CommentPostView
from .. import forms, models

NEW_COMMENT_SUBJECT = apps.get_app_config('groups').new_comment_subject


class DiscussionThread(CommentPostView):
    """Allow a user to read and comment on a Discussion."""
    form_class = forms.AddTextComment
    subscribe_form_class = forms.SubscribeForm
    template_name = 'groups/discussion_thread.html'

    def get_queryset(self):
        """
        Display the comments attached to a given discussion, newest at the bottom.

        Use the CommentManager's with_user_may_delete method to annotate each comment
        with a value denoting if it can be deleted by the current user. This allows us
        to conditionally display the delete link in the template.
        """
        return self.discussion.comments.with_user_may_delete(self.request.user)

    def get_context_data(self, *args, **kwargs):
        """Attach the discussion and its existing comments to the context."""
        context = super(DiscussionThread, self).get_context_data(*args, **kwargs)
        discussion = self.discussion
        form = self.subscribe_form_class(
            user=self.request.user,
            instance=discussion,
            url_name='discussion-subscribe',
        )
        context['comments'] = self.get_queryset()
        context['group'] = discussion.group
        context['discussion-subscribe-form'] = form
        return context


class CommentUploadFile(CommentPostView):
    """Posts a file to a particular discussion."""
    form_class = forms.AddFileComment
    template_name = 'groups/comment_upload_file.html'


class CommentDelete(DeleteView):
    """
    Deletes a particular comment after confirming with an 'Are you sure?' page.

    Rather than fully deleting the comment, set its state to DELETED so it continues to
    exist but displays differently.
    """
    model = models.BaseComment
    template_name = 'groups/comment_delete.html'

    def dispatch(self, request, *args, **kwargs):
        """
        Disallow access to any user other than admins or the comment creator.

        On unauthorised access, kick the user back to the discussion they came from with
        a Django message displayed.  Don't scroll down to the comment, otherwise they
        might not see the message.
        """
        self.comment = self.get_object()
        if not self.comment.may_be_deleted(request.user):
            messages.error(request, 'You do not have permission to delete this comment.')
            return HttpResponseRedirect(self.comment.discussion.get_absolute_url())

        return super(CommentDelete, self).dispatch(request, *args, **kwargs)

    def delete(self, request, *args, **kwargs):
        self.comment.delete_state()
        return HttpResponseRedirect(self.get_success_url())

    def get_success_url(self):
        return self.comment.get_absolute_url()


class CommentPostByEmail(CommentEmailMixin, View):
    """
    Receive comments posted by email and create them in the database.

    This view is intended to be linked to an endpoint that provides an URL kwarg
    'uuid'.  This matches up to an EmailUUID object that stores the discussion and user
    being used.
    """
    @csrf_exempt
    def dispatch(self, request, *args, **kwargs):
        return super(CommentPostByEmail, self).dispatch(request, *args, **kwargs)

    @staticmethod
    def get_uuid_data(uuid):
        """Unwrap the discussion and user data in the UUID string."""
        data = signing.loads(uuid)
        return {
            'discussion': get_object_or_404(models.Discussion, pk=data['discussion_pk']),
            'user': get_object_or_404(get_user_model(), pk=data['user_pk'])
        }

    @staticmethod
    def extract_uuid_from_email(email, request):
        """
        Turn `reply-{uuid}@domain.com` into just `uuid`.

        The UUIDs contain colons, which aren't allowed in email addresses, so they get
        replaced with dollar signs in the `reply-to` address.  We have to undo that here.
        """
        uuid_regex = r'(?P<uuid>[\w\d\-_$]+)'
        regex = r'reply-{}@{}'.format(uuid_regex, get_current_site(request))
        match = re.compile(regex).match(email)
        if not match:
            raise Http404

        return match.group('uuid').replace('$', ':')

    @staticmethod
    def create_file_comments(request, user, discussion):
        """
        Create any number of file comments from the attachments in this message.

        Mailgun provides an entry called `attachment-count` to store the number
        of attachments, then each attachment is a separate entry, `attachment-x` where
        `x` is a number.
        """
        for attachment in request.FILES.values():
            models.FileComment.objects.create(
                file=attachment,
                user=user,
                discussion=discussion,
            )

    def post(self, request, *args, **kwargs):
        """Create a new comment to self.pk."""
        message = request.POST
        uuid = self.extract_uuid_from_email(message['recipient'], request)
        target = self.get_uuid_data(uuid)

        user = target['user']
        discussion = target['discussion']

        self.create_file_comments(request, user, discussion)

        content = message['stripped-text']
        comment = models.TextComment.objects.create(
            body=content,
            user=user,
            discussion=discussion,
        )
        self.email_subscribers(comment)

        return HttpResponse(status=200)



from incuna_test_utils.compat import Python2AssertMixin

from . import factories
from .utils import RequestTestCase
from ..views import groups


class TestGroupList(Python2AssertMixin, RequestTestCase):
    view_class = groups.GroupList

    def test_get(self):
        groups = factories.GroupFactory.create_batch(2)

        request = self.create_request()
        view = self.view_class.as_view()

        response = view(request)
        self.assertEqual(response.status_code, 200)
        object_list = response.context_data['object_list']
        self.assertCountEqual(groups, object_list)


class TestGroupDetail(RequestTestCase):
    view_class = groups.GroupDetail

    def test_get(self):
        group = factories.GroupFactory.create()

        request = self.create_request()
        view = self.view_class.as_view()

        response = view(request, pk=group.pk)
        self.assertEqual(response.status_code, 200)
        detail_object = response.context_data['group']
        self.assertEqual(group, detail_object)



import datetime

from django.contrib.auth.models import User
from django.db import models as django_models
from django.test import TestCase
from incuna_test_utils.compat import Python2AssertMixin

from . import factories
from .. import managers, models


class TestGroupManager(Python2AssertMixin, TestCase):
    def test_discussions(self):
        discussion = factories.DiscussionFactory.create()
        discussion_two = factories.DiscussionFactory.create()

        results = models.Group.objects.discussions()
        self.assertCountEqual([discussion, discussion_two], results)

    def test_comments(self):
        comment = factories.TextCommentFactory.create()
        comment_two = factories.TextCommentFactory.create()

        results = models.Group.objects.comments()
        self.assertCountEqual([comment, comment_two], results)

    def test_users(self):
        comment = factories.TextCommentFactory.create()
        comment_two = factories.TextCommentFactory.create()
        factories.UserFactory.create()

        results = models.Group.objects.users()
        self.assertCountEqual([comment.user, comment_two.user], results)

    def test_users_distinct(self):
        """Assert that Group.objects.users() contains no duplicates."""
        user = factories.UserFactory.create()
        factories.TextCommentFactory.create_batch(2, user=user)

        self.assertCountEqual([user], models.Group.objects.users())

    def test_within_days(self):
        comment = factories.TextCommentFactory.create(date_created=datetime.date.today())
        factories.TextCommentFactory.create(date_created=datetime.date(1970, 1, 1))

        results = models.Group.objects.within_days()
        self.assertCountEqual([comment.discussion.group], results)

    def test_within_time(self):
        comment = factories.TextCommentFactory.create()
        factories.TextCommentFactory.create(date_created=datetime.date(1970, 1, 1))
        delta = datetime.timedelta(hours=6)

        results = models.Group.objects.within_time(delta)
        self.assertCountEqual([comment.discussion.group], results)

    def test_since(self):
        comment = factories.TextCommentFactory.create()
        factories.TextCommentFactory.create(date_created=datetime.date(1970, 1, 1))
        when = datetime.date(2000, 1, 1)

        results = models.Group.objects.since(when)
        self.assertCountEqual([comment.discussion.group], results)

    def test_within_days_distinct(self):
        """Assert that Group.objects.within_days() contains no duplicates."""
        group = factories.GroupFactory.create()
        factories.TextCommentFactory.create_batch(
            2,
            date_created=datetime.date.today(),
            discussion__group=group,
        )

        self.assertCountEqual([group], models.Group.objects.within_days())

    def test_within_time_distinct(self):
        """Assert that Group.objects.within_time() contains no duplicates."""
        group = factories.GroupFactory.create()
        factories.TextCommentFactory.create_batch(2, discussion__group=group)
        delta = datetime.timedelta(hours=6)

        self.assertCountEqual([group], models.Group.objects.within_time(delta))


class TestDiscussionManager(Python2AssertMixin, TestCase):
    def test_for_group(self):
        group = factories.GroupFactory.create()
        discussion = factories.DiscussionFactory.create(group=group)
        factories.DiscussionFactory.create()

        results = models.Discussion.objects.for_group(group)
        self.assertCountEqual([discussion], results)

    def test_comments(self):
        comment = factories.TextCommentFactory.create()
        comment_two = factories.TextCommentFactory.create()

        results = models.Discussion.objects.comments()
        self.assertCountEqual([comment, comment_two], results)

    def test_users(self):
        comment = factories.TextCommentFactory.create()
        comment_two = factories.TextCommentFactory.create()
        factories.UserFactory.create()

        results = models.Discussion.objects.users()
        self.assertCountEqual([comment.user, comment_two.user], results)

    def test_users_distinct(self):
        """Assert that Discussion.objects.users() contains no duplicates."""
        user = factories.UserFactory.create()
        factories.TextCommentFactory.create_batch(2, user=user)

        self.assertCountEqual([user], models.Discussion.objects.users())

    def test_chaining(self):
        """Assert that chaining some of the above methods together works properly."""
        comment = factories.TextCommentFactory.create()
        factories.TextCommentFactory.create()
        group = comment.discussion.group

        results = models.Discussion.objects.for_group(group).comments()
        self.assertCountEqual([comment], results)

    def test_within_days(self):
        comment = factories.TextCommentFactory.create(date_created=datetime.date.today())
        factories.TextCommentFactory.create(date_created=datetime.date(1970, 1, 1))

        results = models.Discussion.objects.within_days()
        self.assertCountEqual([comment.discussion], results)

    def test_within_time(self):
        comment = factories.TextCommentFactory.create()
        factories.TextCommentFactory.create(date_created=datetime.date(1970, 1, 1))
        delta = datetime.timedelta(hours=6)

        results = models.Discussion.objects.within_time(delta)
        self.assertCountEqual([comment.discussion], results)

    def test_since(self):
        comment = factories.TextCommentFactory.create()
        factories.TextCommentFactory.create(date_created=datetime.date(1970, 1, 1))
        when = datetime.date(2000, 1, 1)

        results = models.Discussion.objects.since(when)
        self.assertCountEqual([comment.discussion], results)

    def test_within_days_distinct(self):
        """Assert that Discussion.objects.within_days() contains no duplicates."""
        discussion = factories.DiscussionFactory.create()
        factories.TextCommentFactory.create_batch(
            2,
            date_created=datetime.date.today(),
            discussion=discussion,
        )

        self.assertCountEqual([discussion], models.Discussion.objects.within_days())

    def test_within_time_distinct(self):
        """Assert that Discussion.objects.within_time() contains no duplicates."""
        discussion = factories.DiscussionFactory.create()
        factories.TextCommentFactory.create_batch(2, discussion=discussion)
        delta = datetime.timedelta(hours=6)

        self.assertCountEqual([discussion], models.Discussion.objects.within_time(delta))


class TestCommentManager(Python2AssertMixin, TestCase):
    def test_for_group(self):
        comment = factories.TextCommentFactory.create()
        factories.TextCommentFactory.create()

        results = models.BaseComment.objects.for_group(comment.discussion.group)
        self.assertCountEqual([comment], results)

    def test_for_discussion(self):
        comment = factories.TextCommentFactory.create()
        factories.TextCommentFactory.create()

        results = models.BaseComment.objects.for_discussion(comment.discussion)
        self.assertCountEqual([comment], results)

    def test_users(self):
        user = factories.TextCommentFactory.create().user
        user_two = factories.TextCommentFactory.create().user
        factories.UserFactory.create()

        results = models.BaseComment.objects.users()
        self.assertCountEqual([user, user_two], results)

    def test_users_distinct(self):
        """Assert that BaseComment.objects.users() contains no duplicates."""
        user = factories.UserFactory.create()
        factories.TextCommentFactory.create_batch(2, user=user)

        self.assertCountEqual([user], models.BaseComment.objects.users())

    def test_within_days(self):
        comment = factories.TextCommentFactory.create(date_created=datetime.date.today())
        factories.TextCommentFactory.create(date_created=datetime.date(1970, 1, 1))

        self.assertCountEqual([comment], models.BaseComment.objects.within_days())

    def test_within_time(self):
        comment = factories.TextCommentFactory.create()
        factories.TextCommentFactory.create(date_created=datetime.date(1970, 1, 1))
        delta = datetime.timedelta(hours=6)

        self.assertCountEqual([comment], models.BaseComment.objects.within_time(delta))

    def test_since(self):
        comment = factories.TextCommentFactory.create()
        factories.TextCommentFactory.create(date_created=datetime.date(1970, 1, 1))
        when = datetime.date(2000, 1, 1)

        results = models.BaseComment.objects.since(when)
        self.assertCountEqual([comment], results)

    def test_chaining(self):
        """Assert that chaining some of the above methods together works properly."""
        comment = factories.TextCommentFactory.create()
        factories.TextCommentFactory.create()
        group = comment.discussion.group

        results = models.BaseComment.objects.for_group(group).users()
        self.assertCountEqual([comment.user], results)

    def test_with_user_may_delete(self):
        comment_one = factories.TextCommentFactory.create()
        comment_two = factories.TextCommentFactory.create()

        results = models.BaseComment.objects.with_user_may_delete(comment_one.user)
        self.assertCountEqual([comment_one, comment_two], results)

        may_delete_values = [comment.user_may_delete for comment in results]
        self.assertCountEqual([True, False], may_delete_values)


class TestWithinDaysQuerySetMixin(Python2AssertMixin, TestCase):
    mixin = managers.WithinDaysQuerySetMixin

    def __init__(self, *args, **kwargs):
        """
        Declare some classes that will be used in this test.

        We need a manager and a queryset that inherit from the mixin
        being tested.

        We also set the model attribute on a manager instance to emulate
        adding the manager to a User model.
        """
        super(TestWithinDaysQuerySetMixin, self).__init__(*args, **kwargs)

        class MixedInQuerySet(self.mixin, django_models.QuerySet):
            """A basic QuerySet extending the mixin."""
            since_filter = 'comments__date_created__gte'

        class MixedInManager(self.mixin, django_models.Manager):
            """A basic Manager extending the mixin and using MixedInQuerySet."""
            since_filter = 'comments__date_created__gte'

            def get_queryset(self):
                return MixedInQuerySet(self.model, using=self._db)

        self.manager = MixedInManager()
        self.manager.model = User

    def setUp(self):
        self.recent_user = factories.UserFactory.create()
        factories.TextCommentFactory.create(user=self.recent_user)
        factories.TextCommentFactory.create(date_created=datetime.date(1970, 1, 1))

    def test_within_days_in_manager(self):
        """Assert that within_days() works properly when called from the manager."""
        results = self.manager.within_days()
        self.assertCountEqual([self.recent_user], results)

    def test_within_days_in_queryset(self):
        """Assert that within_days() works properly when called from the queryset."""
        results = self.manager.all().within_days()
        self.assertCountEqual([self.recent_user], results)

    def test_within_time_in_manager(self):
        """Assert that within_time() works properly when called from the manager."""
        results = self.manager.within_time(datetime.timedelta(hours=6))
        self.assertCountEqual([self.recent_user], results)

    def test_within_time_in_queryset(self):
        """Assert that within_time() works properly when called from the queryset."""
        results = self.manager.all().within_time(datetime.timedelta(hours=6))
        self.assertCountEqual([self.recent_user], results)

    def test_since_in_manager(self):
        """Assert that since() works properly when called from the manager."""
        results = self.manager.since(datetime.date(2000, 1, 1))
        self.assertCountEqual([self.recent_user], results)

    def test_since_in_queryset(self):
        """Assert that since() works properly when called from the queryset."""
        results = self.manager.all().since(datetime.date(2000, 1, 1))
        self.assertCountEqual([self.recent_user], results)



import os

import dj_database_url


BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

DEBUG = True
ALLOWED_HOSTS = []
ROOT_URLCONF = 'groups.tests.urls'
STATIC_URL = '/static/'

SECRET_KEY = 'krc34ji^-fd-=+r6e%p!0u0k9h$9!q*_#l=6)74h#o(jrxsx4p'
PASSWORD_HASHERS = ('django.contrib.auth.hashers.MD5PasswordHasher',)

DATABASES = {
    'default': dj_database_url.config(default='postgres://localhost/groups')
}
DEFAULT_FILE_STORAGE = 'inmemorystorage.InMemoryStorage'

INSTALLED_APPS = (
    'groups',
    'groups.tests',

    'crispy_forms',
    'polymorphic',

    # Put contenttypes before auth to work around test issue.
    # See: https://code.djangoproject.com/ticket/10827#comment:12
    'django.contrib.contenttypes',
    'django.contrib.auth',
    'django.contrib.admin',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
)
MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
)

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.contrib.auth.context_processors.auth',
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.core.context_processors.request',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]


CRISPY_TEMPLATE_PACK = 'bootstrap3'

TEST_RUNNER = 'test_project.test_runner.Runner'



import datetime

from django.apps import apps
from django.contrib.auth import get_user_model
from django.db import models
from polymorphic import PolymorphicManager, PolymorphicQuerySet


DEFAULT_WITHIN_DAYS = apps.get_app_config('groups').default_within_days


class WithinDaysQuerySetMixin:
    """
    A mixin that adds methods for returning items that have recently been posted (to).

    Can be mixed into a Manager or a QuerySet.

    Requires a `since_filter` member on the inheriting class, which is the condition used
    by the since() method. To set up the mixin for use in a `User` manager or queryset,
    use:

        since_filter = 'comments__date_created__gte'
    """
    @staticmethod
    def get_threshold_delta(timedelta):
        """Return the earliest posting *time* an item can have and still be recent."""
        return datetime.datetime.now() - timedelta

    @staticmethod
    def get_threshold_date(within_days=DEFAULT_WITHIN_DAYS):
        """Return the earliest posting *date* an item can have and still be recent."""
        return datetime.date.today() - datetime.timedelta(days=within_days)

    def within_days(self, days=DEFAULT_WITHIN_DAYS):
        """All users that created an item within the last `days` days."""
        return self.since(self.get_threshold_date(days))

    def within_time(self, timedelta):
        """All users that created an item within the last `timedelta`."""
        return self.since(self.get_threshold_delta(timedelta))

    def since(self, when):
        """All the comments belonging to items in this queryset posted since `when`."""
        since_filter = {self.since_filter: when}
        return self.filter(**since_filter).distinct()


class GroupQuerySet(WithinDaysQuerySetMixin, models.QuerySet):
    """A queryset for Groups allowing for smarter retrieval of related objects."""
    since_filter = 'discussions__comments__date_created__gte'

    def discussions(self):
        """All the discussions on these groups."""
        from .models import Discussion
        return Discussion.objects.filter(group__in=self)

    def comments(self):
        """All the comments posted in these groups."""
        from .models import BaseComment
        return BaseComment.objects.filter(discussion__group__in=self)

    def users(self):
        """All the users who have ever posted in these groups."""
        User = get_user_model()
        return User.objects.filter(comments__in=self.comments()).distinct()


class DiscussionQuerySet(WithinDaysQuerySetMixin, models.QuerySet):
    """A queryset for Discussions allowing for smarter retrieval of related objects."""
    since_filter = 'comments__date_created__gte'

    def for_group(self, group):
        """All the discussions on a particular group."""
        return self.filter(group=group)

    def comments(self):
        """All the comments on these discussions."""
        from .models import BaseComment
        return BaseComment.objects.filter(discussion__in=self)

    def users(self):
        """All the users who have ever posted in these discussions."""
        User = get_user_model()
        return User.objects.filter(comments__discussion__in=self).distinct()


class CommentManagerMixin(WithinDaysQuerySetMixin):
    """
    Provides methods suitable for use on both a queryset and a manager for BaseComments.

    django-polymorphic's managers/querysets don't play well with related managers, so we
    have to force its hand a bit.

    Based on https://djangosnippets.org/snippets/2114/.
    """
    since_filter = 'date_created__gte'

    def for_group(self, group):
        """All the comments on a particular group."""
        return self.filter(discussion__group=group)

    def for_discussion(self, discussion):
        """All the comments on a particular discussion."""
        return self.filter(discussion=discussion)

    def users(self):
        """All the users who, between them, posted these comments."""
        User = get_user_model()
        return User.objects.filter(comments__in=self.all()).distinct()

    def with_user_may_delete(self, user):
        """
        Return a list of comments annotated with 'user_may_delete' values.

        The value denotes if the passed-in user is allowed to delete this particular
        comment. The method returns a list instead of a QuerySet to avoid removing the
        added variable with further filters.
        """
        comments = list(self.all())
        for comment in comments:
            comment.user_may_delete = comment.may_be_deleted(user)

        return comments


class CommentQuerySet(PolymorphicQuerySet, CommentManagerMixin):
    """A queryset for BaseComments allowing for smarter retrieval of related objects."""


class CommentManager(PolymorphicManager, CommentManagerMixin):
    """PolymorphicManager for BaseComments with custom methods."""
    def get_queryset(self):
        return CommentQuerySet(self.model, using=self._db)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('groups', '0014_discussion_ignorers'),
    ]

    operations = [
        migrations.AlterField(
            model_name='discussion',
            name='ignorers',
            field=models.ManyToManyField(to=settings.AUTH_USER_MODEL, blank=True, related_name='ignored_discussions'),
        ),
        migrations.AlterField(
            model_name='discussion',
            name='subscribers',
            field=models.ManyToManyField(to=settings.AUTH_USER_MODEL, blank=True, related_name='subscribed_discussions'),
        ),
        migrations.AlterField(
            model_name='group',
            name='members_if_private',
            field=models.ManyToManyField(to=settings.AUTH_USER_MODEL, blank=True, related_name='private_groups_joined'),
        ),
        migrations.AlterField(
            model_name='group',
            name='moderators',
            field=models.ManyToManyField(to=settings.AUTH_USER_MODEL, blank=True, related_name='moderated_groups'),
        ),
        migrations.AlterField(
            model_name='group',
            name='watchers',
            field=models.ManyToManyField(to=settings.AUTH_USER_MODEL, blank=True, related_name='watched_groups'),
        ),
    ]



from incuna_test_utils.testcases.urls import URLTestCase

from ..views import comments, discussions, groups, subscriptions


class TestGroupUrls(URLTestCase):
    pk = 42

    def test_group_list(self):
        self.assert_url_matches_view(
            groups.GroupList,
            '/groups/',
            'group-list',
        )

    def test_group_detail(self):
        self.assert_url_matches_view(
            groups.GroupDetail,
            '/groups/{}/'.format(self.pk),
            'group-detail',
            url_kwargs={'pk': self.pk}
        )

    def test_group_subscribe(self):
        self.assert_url_matches_view(
            subscriptions.GroupSubscribe,
            '/groups/{}/subscribe/'.format(self.pk),
            'group-subscribe',
            url_kwargs={'pk': self.pk}
        )

    def test_discussion_create(self):
        self.assert_url_matches_view(
            discussions.DiscussionCreate,
            '/groups/{}/new-discussion/'.format(self.pk),
            'discussion-create',
            url_kwargs={'pk': self.pk}
        )

    def test_discussion_detail(self):
        self.assert_url_matches_view(
            discussions.DiscussionThread,
            '/groups/discussions/{}/'.format(self.pk),
            'discussion-thread',
            url_kwargs={'pk': self.pk}
        )

    def test_discussion_subscribe(self):
        self.assert_url_matches_view(
            subscriptions.DiscussionSubscribe,
            '/groups/discussions/{}/subscribe/'.format(self.pk),
            'discussion-subscribe',
            url_kwargs={'pk': self.pk}
        )

    def test_comment_upload_file(self):
        self.assert_url_matches_view(
            comments.CommentUploadFile,
            '/groups/discussions/{}/upload-file/'.format(self.pk),
            'comment-upload-file',
            url_kwargs={'pk': self.pk}
        )

    def test_comment_delete(self):
        self.assert_url_matches_view(
            comments.CommentDelete,
            '/groups/comments/{}/delete/'.format(self.pk),
            'comment-delete',
            url_kwargs={'pk': self.pk}
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('groups', '0004_modify_comment_fields'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='comment',
            options={'ordering': ('date_created',)},
        ),
    ]



from django.apps import apps
from django.core.urlresolvers import reverse
from django.shortcuts import get_object_or_404
from django.views.generic import FormView
from incuna_mail import send

from ._helpers import CommentPostView, get_reply_address
from .. import forms, models

NEW_DISCUSSION_SUBJECT = apps.get_app_config('groups').new_discussion_subject


class DiscussionCreate(FormView):
    """Start a new discussion by writing its first comment."""
    form_class = forms.DiscussionCreate
    template_name = 'groups/discussion_form.html'

    def get_group(self):
        return get_object_or_404(models.Group, pk=self.kwargs['pk'])

    def get_context_data(self, *args, **kwargs):
        context = super(DiscussionCreate, self).get_context_data(*args, **kwargs)
        context['group'] = self.get_group()
        return context

    def get_success_url(self):
        return reverse('discussion-thread', kwargs={'pk': self.pk})

    def form_valid(self, form):
        user = self.request.user
        discussion = models.Discussion.objects.create(
            creator=user,
            group=self.get_group(),
            name=form.cleaned_data['name'],
        )
        discussion.subscribers.add(user)
        models.TextComment.objects.create(
            body=form.cleaned_data['comment'],
            discussion=discussion,
            user=user,
        )
        self.pk = discussion.pk
        self.email_subscribers(discussion)
        return super(DiscussionCreate, self).form_valid(form)

    def email_subscribers(self, discussion):
        """Notify all subscribers to the discussion's parent group, except its creator."""
        users = discussion.group.watchers.exclude(pk=self.request.user.pk)
        for user in users:
            send(
                to=user.email,
                subject=NEW_DISCUSSION_SUBJECT.format(group=discussion.group.name),
                template_name='groups/emails/new_discussion.txt',
                reply_to=get_reply_address(discussion, user, self.request),
                context={
                    'discussion': discussion,
                    'user': user,
                },
            )


class DiscussionThread(CommentPostView):
    """Allow a user to read and comment on a Discussion."""
    form_class = forms.AddTextComment
    subscribe_form_class = forms.SubscribeForm
    template_name = 'groups/discussion_thread.html'

    def get_queryset(self):
        """
        Display the comments attached to a given discussion, newest at the bottom.

        Use the CommentManager's with_user_may_delete method to annotate each comment
        with a value denoting if it can be deleted by the current user. This allows us
        to conditionally display the delete link in the template.
        """
        return self.discussion.comments.with_user_may_delete(self.request.user)

    def get_context_data(self, *args, **kwargs):
        """Attach the discussion and its existing comments to the context."""
        context = super(DiscussionThread, self).get_context_data(*args, **kwargs)
        discussion = self.discussion
        form = self.subscribe_form_class(
            user=self.request.user,
            instance=discussion,
            url_name='discussion-subscribe',
        )
        context['comments'] = self.get_queryset()
        context['group'] = discussion.group
        context['discussion-subscribe-form'] = form
        return context



try:
    from unittest import mock
except ImportError:
    import mock

from django.contrib.sites.shortcuts import get_current_site
from django.core import mail
from incuna_test_utils.compat import Python2AssertMixin

from . import factories
from .utils import RequestTestCase
from ..views import _helpers as helpers


class TestGetReplyAddress(Python2AssertMixin, RequestTestCase):
    def test_get_reply_address(self):
        """
        Assert that the method returns `reply-{uuid}@{domain}`.

        Look for dollar signs ($) instead of colons in the UUID, because we've done some
        replace work to ensure the email address is legal.
        """
        request = self.create_request()
        discussion = factories.DiscussionFactory.create()

        domain = get_current_site(request).domain
        uuid_regex = r'[\d\w\-_$]*'  # A string of alphanumerics, `-`, `_`, and/or `$`
        self.assertRegex(
            helpers.get_reply_address(discussion, request.user, request),
            r'reply-{uuid}@{domain}'.format(uuid=uuid_regex, domain=domain)
        )


class TestCommentEmailMixin(RequestTestCase):
    def setUp(self):
        """Instantiate a minimal CommentEmailMixin object."""
        self.discussion = factories.DiscussionFactory.create()
        self.request = self.create_request()
        self.view_obj = helpers.CommentEmailMixin()
        self.view_obj.request = self.request

    def test_users_to_notify(self):
        """
        Test that users_to_notify picks the right users.

        * All subscribers to the discussion,
        * plus all subscribers to the discussion's parent group,
        * minus everyone who ignored the discussion,
        * minus the user who posted the comment.
        """
        (
            group_subscriber,  # Will be notified.
            discussion_subscriber,  # Will also be notified.
            discussion_ignorer,  # A group subscriber, but ignores the discussion = no.
            comment_poster,  # The poster isn't notified regardless of subscriptions.
            unrelated_user,  # Not subscribed at all = no notifications.
        ) = factories.UserFactory.create_batch(5)

        group = factories.GroupFactory.create()
        discussion = factories.DiscussionFactory.create(group=group)
        comment = factories.BaseCommentFactory.create(
            user=comment_poster,
            discussion=discussion,
        )

        # Set up the various subscription preferences as described above.
        group.watchers = [group_subscriber, discussion_ignorer, comment_poster]
        discussion.subscribers = [discussion_subscriber, comment_poster]
        discussion.ignorers = [discussion_ignorer]

        users = helpers.CommentPostView.users_to_notify(comment)
        self.assertEqual(set(users), {group_subscriber, discussion_subscriber})

    def test_email_subscribers(self):
        """
        Test notification emails for a new comment.

        The users_to_notify logic is tested above, so we can mock it here.  Same with
        get_reply_address, which is awkward to assert otherwise.
        """
        subscriber = factories.UserFactory.create()
        discussion = factories.DiscussionFactory.create()
        comment = factories.TextCommentFactory.create(discussion=discussion)

        reply_address = 'leeroy@jenkins.com'
        users_method_path = 'groups.views._helpers.CommentEmailMixin.users_to_notify'
        address_method_path = 'groups.views._helpers.get_reply_address'
        with mock.patch(users_method_path, return_value=[subscriber]):
            with mock.patch(address_method_path, return_value=reply_address):
                self.view_obj.email_subscribers(comment)

        self.assertEqual(len(mail.outbox), 1)
        email = mail.outbox[0]
        self.assertEqual(email.subject, 'New comment on {}'.format(discussion.name))
        self.assertEqual(email.to, [subscriber.email])
        self.assertEqual(email.reply_to, [reply_address])
        self.assertIn('A new comment has been posted', email.body)
        self.assertIn(subscriber.get_full_name(), email.body)
        self.assertIn(comment.body, email.body)


class TestCommentPostView(Python2AssertMixin, RequestTestCase):
    def setUp(self):
        """Instantiate a minimal CommentPostView object."""
        class TestView(helpers.CommentPostView):
            """Set fields attribute to fulfil ModelFormMixin requirements."""
            fields = ()

        self.discussion = factories.DiscussionFactory.create()
        self.request = self.create_request()
        self.view_obj = TestView(
            request=self.request,
            kwargs={'pk': self.discussion.pk},
        )

        self.view_obj.dispatch(self.request)

    def test_dispatch(self):
        """After dispatch (called during setUp) the discussion is attached to the view."""
        self.assertEqual(self.view_obj.discussion, self.discussion)

    def test_includes_discussion(self):
        """Assert that the discussion is picked up and output."""
        context_data = self.view_obj.get_context_data()
        self.assertEqual(context_data['discussion'], self.discussion)

    def test_form_valid(self):
        """Assert that the request user and discussion are attached to the instance."""
        form = mock.MagicMock(instance=mock.MagicMock())
        self.view_obj.form_valid(form)
        self.assertEqual(form.instance.user, self.request.user)
        self.assertEqual(form.instance.discussion, self.discussion)



from django.core.urlresolvers import reverse
from django_webtest import WebTest

from . import factories
from .utils import RequestTestCase
from ..views import subscriptions


class TestDiscussionSubscribe(RequestTestCase):
    view_class = subscriptions.DiscussionSubscribe

    def test_form_subscribe(self):
        discussion = factories.DiscussionFactory.create()
        user = self.user_factory.create()

        # Hit the view, passing in the necessaries.
        request = self.create_request('post', user=user, data={'subscribe': True})
        view = self.view_class.as_view()
        view(request, pk=discussion.pk)

        # Assert that the user is now subscribed.
        self.assertIn(user, discussion.subscribers.all())
        self.assertIn(discussion, user.subscribed_discussions.all())

    def test_form_unsubscribe(self):
        discussion = factories.DiscussionFactory.create()
        user = self.user_factory.create()

        # Subscribe the user.
        discussion.subscribers.add(user)

        # Hit the view, passing in the necessaries.
        request = self.create_request('post', user=user, data={'subscribe': False})
        view = self.view_class.as_view()
        view(request, pk=discussion.pk)

        # Assert that the user is now unsubscribed.
        self.assertNotIn(user, discussion.subscribers.all())
        self.assertNotIn(discussion, user.subscribed_discussions.all())


class TestGroupSubscribe(RequestTestCase):
    def setUp(self):
        self.view = subscriptions.GroupSubscribe.as_view()

    def test_form_subscribe(self):
        group = factories.GroupFactory.create()
        request = self.create_request('post', data={'subscribe': True})

        response = self.view(request, pk=group.pk)

        self.assertEqual(response.status_code, 302)
        self.assertIn(request.user, group.watchers.all())
        self.assertIn(group, request.user.watched_groups.all())

    def test_form_unsubscribe(self):
        group = factories.GroupFactory.create()
        request = self.create_request('post', data={'subscribe': False})
        group.watchers.add(request.user)

        response = self.view(request, pk=group.pk)

        self.assertEqual(response.status_code, 302)
        self.assertNotIn(request.user, group.watchers.all())
        self.assertNotIn(group, request.user.watched_groups.all())


class TestGroupSubscribeIntegration(WebTest):
    def setUp(self):
        self.user = factories.UserFactory.create()
        self.group = factories.GroupFactory.create()
        self.group_url = reverse('group-detail', kwargs={'pk': self.group.pk})

    def test_subscribe_unsubscribe(self):
        """A user can subscribe and unsubscribe from a group."""
        # A user is subscribed to the group if they click the button
        self.app.get(self.group_url, user=self.user).form.submit()
        self.assertIn(self.user, self.group.watchers.all())

        # A user is unsubscribed from the group if they click the button
        self.app.get(self.group_url, user=self.user).form.submit()
        self.assertNotIn(self.user, self.group.watchers.all())



import factory
from django.contrib.auth import get_user_model
from incuna_test_utils.factories import images

from .. import models


class UserFactory(factory.DjangoModelFactory):
    username = factory.Sequence('user{}'.format)
    first_name = factory.Sequence('Leeroy{}'.format)
    last_name = factory.Sequence('Jenkins{}'.format)
    email = factory.Sequence('email{}@example.com'.format)
    is_active = True

    class Meta:
        model = get_user_model()

    @factory.post_generation
    def password(self, create, extracted, **kwargs):
        # By using this method, password can never be set to `None`!
        self.raw_password = 'password' if extracted is None else extracted
        self.set_password(self.raw_password)
        if create:
            self.save()


class AdminFactory(UserFactory):
    is_staff = True
    is_superuser = True


class GroupFactory(factory.DjangoModelFactory):
    name = factory.Sequence('Group {}'.format)

    class Meta:
        model = models.Group


class DiscussionFactory(factory.DjangoModelFactory):
    name = factory.Sequence('Discussion {}'.format)
    group = factory.SubFactory(GroupFactory)
    creator = factory.SubFactory(UserFactory)

    class Meta:
        model = models.Discussion


class BaseCommentFactory(factory.DjangoModelFactory):
    discussion = factory.SubFactory(DiscussionFactory)
    user = factory.SubFactory(UserFactory)

    class Meta:
        model = models.BaseComment


class TextCommentFactory(BaseCommentFactory):
    body = factory.Sequence('Comment {}'.format)

    class Meta:
        model = models.TextComment


class FileCommentFactory(BaseCommentFactory):
    file = images.LocalFileField()

    class Meta:
        model = models.FileComment



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('groups', '0007_change_ordering'),
    ]

    operations = [
        migrations.AddField(
            model_name='comment',
            name='state',
            field=models.CharField(max_length=255, default='ok', choices=[('ok', 'OK'), ('deleted', 'Deleted')]),
            preserve_default=True,
        ),
    ]



from django.shortcuts import get_object_or_404
from django.views.generic import ListView

from .. import forms, models


class GroupList(ListView):
    """Show a top-level list of discussion groups."""
    model = models.Group
    paginate_by = 5
    template_name = 'groups/group_list.html'


class GroupDetail(ListView):
    """Show the discussions belonging to a group."""
    model = models.Discussion
    paginate_by = 5
    template_name = 'groups/group_detail.html'
    subscribe_form_class = forms.SubscribeForm

    def get_queryset(self):
        return super(GroupDetail, self).get_queryset().filter(group=self.group)

    def get_context_data(self, *args, **kwargs):
        context = super(GroupDetail, self).get_context_data(*args, **kwargs)
        context['group'] = self.group
        context['group-subscribe-form'] = self.subscribe_form_class(
            user=self.request.user,
            instance=self.group,
            url_name='group-subscribe',
        )
        return context

    def dispatch(self, request, *args, **kwargs):
        self.group = get_object_or_404(models.Group, pk=self.kwargs['pk'])
        return super(GroupDetail, self).dispatch(request, *args, **kwargs)



from django.apps import apps as django_apps
from django.test import TestCase


class TestGroupsConfig(TestCase):
    config = django_apps.get_app_config('groups')

    def test_default_within_days(self):
        """Assert this member exists and has the right value."""
        value = self.config.default_within_days
        self.assertEqual(value, 7)



try:
    from unittest import mock
except ImportError:
    import mock

import datetime

import pytz
from django.contrib.sites.shortcuts import get_current_site
from django.core import signing
from django.http import Http404
from incuna_test_utils.compat import Python2AssertMixin

from . import factories
from .utils import RequestTestCase
from .. import models
from ..views import comments


class TestCommentUploadFile(Python2AssertMixin, RequestTestCase):
    view_class = comments.CommentUploadFile

    def make_datetime(self, year, month, day):
        return datetime.datetime(year, month, day, tzinfo=pytz.utc)

    def test_get(self):
        discussion = factories.DiscussionFactory.create()
        request = self.create_request()
        view = self.view_class.as_view()

        response = view(request, pk=discussion.pk)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context_data['discussion'], discussion)

    def test_post(self):
        discussion = factories.DiscussionFactory.create()
        file = factories.FileCommentFactory.build().file.file
        data = {'file': file}

        # Hit the view, passing in the necessaries.
        request = self.create_request('post', data=data)
        view = self.view_class.as_view()
        view(request, pk=discussion.pk)

        # Assert that one comment was created with the appropriate properties.
        created_comment = models.FileComment.objects.get(discussion=discussion)
        self.assertEqual(created_comment.discussion, discussion)
        self.assertEqual(created_comment.user, request.user)


class TestCommentDelete(RequestTestCase):
    view_class = comments.CommentDelete

    def setUp(self):
        self.comment = factories.TextCommentFactory.create()
        self.request = self.create_request(user=self.comment.user)

    def test_get(self):
        view = self.view_class.as_view()
        response = view(self.request, pk=self.comment.pk)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context_data['object'], self.comment)

    def test_get_forbidden(self):
        """A user who isn't allowed to delete the comment sees an error message."""
        view = self.view_class.as_view()
        request = self.create_request()
        response = view(request, pk=self.comment.pk)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['location'], self.comment.discussion.get_absolute_url())
        self.assertEqual(
            'You do not have permission to delete this comment.',
            request._messages.store[0]
        )

    def test_post(self):
        """POSTing to this endpoint deletes the comment."""
        view = self.view_class.as_view()
        self.request.method = 'post'
        response = view(self.request, pk=self.comment.pk)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['location'], self.comment.get_absolute_url())

        updated_comment = models.TextComment.objects.get(pk=self.comment.pk)
        self.assertTrue(updated_comment.is_deleted())

    def test_post_forbidden(self):
        """A user who isn't allowed to delete the comment sees an error message."""
        view = self.view_class.as_view()
        request = self.create_request('post')
        response = view(request, pk=self.comment.pk)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['location'], self.comment.discussion.get_absolute_url())
        self.assertEqual(
            'You do not have permission to delete this comment.',
            request._messages.store[0]
        )

    def test_delete(self):
        view_obj = self.view_class(request=self.request, kwargs={'pk': self.comment.pk})
        view_obj.comment = self.comment

        view_obj.delete(self.request)
        self.assertTrue(self.comment.is_deleted())

    def test_get_success_url(self):
        view_obj = self.view_class(request=self.request, kwargs={'pk': self.comment.pk})
        view_obj.comment = self.comment

        expected = self.comment.get_absolute_url()
        self.assertEqual(view_obj.get_success_url(), expected)


class TestCommentPostByEmail(RequestTestCase):
    view_class = comments.CommentPostByEmail

    # Some method paths for mocking out helpers.
    extract_path = 'groups.views.comments.CommentPostByEmail.extract_uuid_from_email'
    uuid_path = 'groups.views.comments.CommentPostByEmail.get_uuid_data'
    email_path = 'groups.views.comments.CommentEmailMixin.email_subscribers'
    file_create_path = 'groups.views.comments.CommentPostByEmail.create_file_comments'

    def generate_uuid(self, discussion_pk, user_pk):
        """Generate a UUID in the same way as a discussion."""
        data = {'discussion_pk': discussion_pk, 'user_pk': user_pk}
        return signing.dumps(data)

    def test_get_uuid_data(self):
        discussion = factories.DiscussionFactory.create()
        user = discussion.creator
        uuid = self.generate_uuid(discussion.pk, user.pk)

        data = self.view_class.get_uuid_data(uuid)
        expected_data = {'discussion': discussion, 'user': user}
        self.assertEqual(data, expected_data)

    def test_get_uuid_data_no_discussion(self):
        user = factories.UserFactory.create()
        uuid = self.generate_uuid(42, user.pk)

        with self.assertRaises(Http404):
            self.view_class.get_uuid_data(uuid)

    def test_get_uuid_data_no_user(self):
        discussion = factories.DiscussionFactory.create()
        uuid = self.generate_uuid(discussion.pk, 42)

        with self.assertRaises(Http404):
            self.view_class.get_uuid_data(uuid)

    def test_extract_uuid_from_email(self):
        """
        Assert that `reply-{uuid}@{domain}` becomes `uuid`.

        The UUID that is returned has had its dollar signs turned back into colons so
        that it can be parsed properly.
        """
        reply_uuid = 'I-aM$an_UU1D'
        expected_uuid = reply_uuid.replace('$', ':')
        request = self.create_request()
        domain = get_current_site(request).domain
        email = 'reply-{}@{}'.format(reply_uuid, domain)

        extracted_uuid = self.view_class.extract_uuid_from_email(email, request)
        self.assertEqual(extracted_uuid, expected_uuid)

    def test_extract_uuid_failure(self):
        """The method throws a 404 when it fails."""
        with self.assertRaises(Http404):
            self.view_class.extract_uuid_from_email(
                'will-this-work@example.com',
                self.create_request()
            )

    def test_create_file_comments(self):
        """Assert that a file comment is created for each attachment in request.FILES."""
        discussion = factories.DiscussionFactory.create()
        user = discussion.creator

        # Let's use mocking and trust that Manager.create() works rather than generate
        # an actual file.
        file = "I am a file and I'm digging a hole"
        request = mock.MagicMock(FILES={'attachment-1': file})

        with mock.patch.object(models.FileComment.objects, 'create') as file_create:
            self.view_class.create_file_comments(request, user, discussion)

        file_create.assert_called_once_with(file=file, user=user, discussion=discussion)

    def test_post(self):
        discussion = factories.DiscussionFactory.create()
        user = discussion.creator
        uuid_data = {'discussion': discussion, 'user': user}

        message_body = 'Email replying is so straightforward and fun'
        request_data = {
            'stripped-text': message_body,
            'recipient': 'this is needed, but mocked out',
        }

        request = self.create_request(
            method='post',
            content_type='application/json',
        )
        request.POST = request_data
        view = self.view_class.as_view()

        # Mock out all the helper methods, since they're tested separately, and it saves
        # on creating a *ton* of extra setup data.  There doesn't appear to be a way to
        # achieve this without extremely nested `with` statements.
        with mock.patch(self.extract_path, return_value='uuid') as extract_uuid:
            with mock.patch(self.uuid_path, return_value=uuid_data):
                with mock.patch(self.email_path) as email_subscribers:
                    with mock.patch(self.file_create_path) as create_file_comments:
                        response = view(request, uuid='use of this is mocked out')

        # Mailgun likes to receive confirmation, so it's important we send a HTTP200 to
        # avoid resends.
        self.assertEqual(response.status_code, 200)

        # Assert that a comment was created correctly.
        comment = models.TextComment.objects.get()
        self.assertEqual(comment.body, message_body)

        # Assert that we've respected the recipient when generating the UUID, created
        # file comments out of any attachments, and emailed any subscribers.
        extract_uuid.assert_called_once_with(request_data['recipient'], request)
        create_file_comments.assert_called_once_with(request, user, discussion)
        email_subscribers.assert_called_once_with(comment)



from django.apps import apps
from django.contrib.sites.shortcuts import get_current_site
from django.views.generic import CreateView
from incuna_mail import send

from .. import models

NEW_COMMENT_SUBJECT = apps.get_app_config('groups').new_comment_subject


def get_reply_address(discussion, user, request):
    """
    Wrap a discussion reply UUID in an email address suitable for use as a reply-to.

    reply-{uuid}@{domain}

    The UUID contains colons, which aren't allowed in email addresses, so swap those
    out for dollar signs ($) as a placeholder using a string `replace`.  The rest of
    the UUID is base64-encoded, so there will be neither colons nor dollar signs in it
    (it's only alphanumerics, hyphens, and underscores).
    """
    uuid = discussion.generate_reply_uuid(user).replace(':', '$')
    site = get_current_site(request)
    return 'reply-{}@{}'.format(uuid, site)


class CommentEmailMixin:
    """A mixin for CreateViews and similar that build comments."""
    @staticmethod
    def users_to_notify(comment):
        """
        Return subscribers to the comment's discussion or its parent group.

        Exclude anyone who's explicitly ignored this discussion, and the person who
        posted the comment.
        """
        discussion = comment.discussion
        discussion_subscribers = discussion.subscribers.all()
        group_subscribers = discussion.group.watchers.exclude(
            ignored_discussions=discussion
        )

        all_subscribers = discussion_subscribers | group_subscribers
        return all_subscribers.exclude(pk=comment.user.pk)

    def email_subscribers(self, comment):
        """Notify all subscribers to the discussion or its group, except the poster."""
        users = self.users_to_notify(comment)
        for user in users:
            send(
                to=user.email,
                subject=NEW_COMMENT_SUBJECT.format(discussion=comment.discussion.name),
                template_name='groups/emails/new_comment.txt',
                reply_to=get_reply_address(comment.discussion, user, self.request),
                context={
                    'comment': comment,
                    'user': user,
                },
            )


class CommentPostView(CommentEmailMixin, CreateView):
    """
    Base class for views that post a comment to a particular discussion.

    Must be initialised with form_class and template_name attributes, as required by
    the superclass, CreateView.
    """
    model = models.BaseComment

    def dispatch(self, request, *args, **kwargs):
        pk = self.kwargs['pk']
        self.discussion = models.Discussion.objects.select_related('group').get(pk=pk)
        return super(CommentPostView, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, *args, **kwargs):
        """Attach the discussion and its existing comments to the context."""
        context = super(CommentPostView, self).get_context_data(*args, **kwargs)
        context['discussion'] = self.discussion
        return context

    def form_valid(self, form):
        form.instance.user = self.request.user
        form.instance.discussion = self.discussion
        self.email_subscribers(form.instance)
        return super(CommentPostView, self).form_valid(form)



from django.core import signing
from django.test import TestCase
from incuna_test_utils.compat import Python2AssertMixin

from . import factories
from .. import models


class TestGroup(Python2AssertMixin, TestCase):
    def test_fields(self):
        fields = models.Group._meta.get_all_field_names()
        expected = {
            'id',
            'name',
            'is_private',
            'moderators',
            'watchers',
            'members_if_private',

            # From Discussion
            'discussions',
        }
        self.assertCountEqual(fields, expected)

    def test_get_absolute_url(self):
        group = factories.GroupFactory.create()
        expected = '/groups/{}/'.format(group.pk)
        self.assertEqual(group.get_absolute_url(), expected)

    def test_str(self):
        group = factories.GroupFactory.create()
        self.assertEqual(str(group), group.name)


class TestDiscussion(Python2AssertMixin, TestCase):
    def test_fields(self):
        fields = models.Discussion._meta.get_all_field_names()
        expected = [
            'id',
            'name',
            'group',
            'group_id',
            'creator',
            'creator_id',
            'date_created',
            'subscribers',
            'ignorers',

            # From BaseComment
            'comments',
        ]
        self.assertCountEqual(fields, expected)

    def test_get_absolute_url(self):
        discussion = factories.DiscussionFactory.create()
        expected = '/groups/discussions/{}/'.format(discussion.pk)
        self.assertEqual(discussion.get_absolute_url(), expected)

    def test_ordering(self):
        discussion_one, discussion_two = factories.DiscussionFactory.create_batch(2)

        discussions = models.Discussion.objects.all()
        expected = [discussion_two, discussion_one]
        self.assertSequenceEqual(discussions, expected)

    def test_str(self):
        discussion = factories.DiscussionFactory.create()
        self.assertEqual(str(discussion), discussion.name)

    def test_generate_reply_uuid(self):
        discussion = factories.DiscussionFactory.create()
        user = discussion.creator
        expected_data = {
            'discussion_pk': discussion.pk,
            'user_pk': user.pk,
        }

        signed_data = discussion.generate_reply_uuid(user)
        self.assertEqual(signing.loads(signed_data), expected_data)


class TestBaseComment(Python2AssertMixin, TestCase):
    def test_fields(self):
        fields = models.BaseComment._meta.get_all_field_names()
        expected = [
            'id',
            'discussion',
            'discussion_id',
            'user',
            'user_id',
            'date_created',
            'state',

            'polymorphic_ctype',
            'polymorphic_ctype_id',
            'textcomment',
            'filecomment',
        ]
        self.assertCountEqual(fields, expected)

    def test_get_pagejump_anchor(self):
        comment = factories.TextCommentFactory.create()
        expected = 'c{}'.format(comment.pk)
        self.assertEqual(comment.get_pagejump_anchor(), expected)

    def test_get_pagejump(self):
        comment = factories.TextCommentFactory.create()
        expected = '#c{}'.format(comment.pk)
        self.assertEqual(comment.get_pagejump(), expected)

    def test_get_absolute_url(self):
        comment = factories.TextCommentFactory.create()
        expected = '/groups/discussions/{}/#c{}'.format(comment.discussion.pk, comment.pk)
        self.assertEqual(comment.get_absolute_url(), expected)

    def test_may_be_deleted_comment_user(self):
        comment = factories.TextCommentFactory.create()
        self.assertTrue(comment.may_be_deleted(comment.user))

    def test_may_be_deleted_admin(self):
        comment = factories.TextCommentFactory.create()
        admin = factories.AdminFactory.create()
        self.assertTrue(comment.may_be_deleted(admin))

    def test_may_be_deleted_other_user(self):
        comment = factories.TextCommentFactory.create()
        user = factories.UserFactory.create()
        self.assertFalse(comment.may_be_deleted(user))

    def test_may_be_deleted_already_deleted(self):
        comment = factories.TextCommentFactory.create()
        comment.delete_state()
        self.assertFalse(comment.may_be_deleted(comment.user))

    def test_delete_state(self):
        comment = factories.TextCommentFactory.create()
        self.assertEqual(comment.state, comment.STATE_OK)
        comment.delete_state()
        self.assertEqual(comment.state, comment.STATE_DELETED)

    def test_is_deleted(self):
        comment = factories.TextCommentFactory.create()
        self.assertFalse(comment.is_deleted())
        comment.delete_state()
        self.assertTrue(comment.is_deleted())

    def test_str(self):
        comment = factories.TextCommentFactory.create()
        self.assertEqual(
            str(comment),
            '{} on Discussion #{}'.format(
                'TextComment',
                comment.discussion_id
            )
        )


class TestTextComment(Python2AssertMixin, TestCase):
    def test_fields(self):
        fields = models.TextComment._meta.get_all_field_names()
        expected = [
            'id',
            'body',
            'discussion',
            'discussion_id',
            'user',
            'user_id',
            'date_created',
            'state',

            'polymorphic_ctype',
            'polymorphic_ctype_id',
            'basecomment_ptr',
            'basecomment_ptr_id',
        ]
        self.assertCountEqual(fields, expected)


class TestFileComment(Python2AssertMixin, TestCase):
    def test_fields(self):
        fields = models.FileComment._meta.get_all_field_names()
        expected = [
            'id',
            'file',
            'discussion',
            'discussion_id',
            'user',
            'user_id',
            'date_created',
            'state',

            'polymorphic_ctype',
            'polymorphic_ctype_id',
            'basecomment_ptr',
            'basecomment_ptr_id',
        ]
        self.assertCountEqual(fields, expected)

    def test_short_filename(self):
        filename = '/groups/file_comments/name.txt'
        comment = factories.FileCommentFactory.create(file__filename=filename)
        self.assertEqual(comment.short_filename(), 'name.txt')



from django.apps import AppConfig


class GroupsConfig(AppConfig):
    """
    Configuration for `groups`.

    Provides:
    * default_within_days - a default parameter for the `within_days` methods on some
      of the model managers, which return items that were posted or posted to within
      that time period.
    """
    name = 'groups'
    default_within_days = 7

    new_comment_subject = 'New comment on {discussion}'
    new_discussion_subject = 'New discussion in {group}'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Group',
            fields=[
                ('id', models.AutoField(serialize=False, auto_created=True, verbose_name='ID', primary_key=True)),
                ('name', models.CharField(max_length=255)),
                ('is_private', models.BooleanField(default=False)),
                ('members_if_private', models.ManyToManyField(to=settings.AUTH_USER_MODEL)),
                ('moderators', models.ManyToManyField(to=settings.AUTH_USER_MODEL)),
                ('watchers', models.ManyToManyField(to=settings.AUTH_USER_MODEL)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
    ]



try:
    from unittest import mock
except ImportError:
    import mock

import datetime

import pytz
from django.core import mail
from django.core.urlresolvers import reverse
from incuna_test_utils.compat import Python2AssertMixin

from . import factories
from .utils import RequestTestCase
from .. import models
from ..views import discussions


class TestDiscussionThread(Python2AssertMixin, RequestTestCase):
    view_class = discussions.DiscussionThread

    def make_datetime(self, year, month, day):
        return datetime.datetime(year, month, day, tzinfo=pytz.utc)

    def test_get(self):
        discussion = factories.DiscussionFactory.create()
        comment = factories.TextCommentFactory.create(discussion=discussion)
        factories.TextCommentFactory.create()

        request = self.create_request()
        view = self.view_class.as_view()

        response = view(request, pk=discussion.pk)
        self.assertEqual(response.status_code, 200)
        self.assertCountEqual(response.context_data['comments'], [comment])
        self.assertEqual(response.context_data['discussion'], discussion)

    def test_sorting(self):
        discussion = factories.DiscussionFactory.create()
        newer_comment = factories.TextCommentFactory.create(
            discussion=discussion,
            date_created=self.make_datetime(2014, 1, 3)
        )
        older_comment = factories.TextCommentFactory.create(
            discussion=discussion,
            date_created=self.make_datetime(2014, 1, 1)
        )

        request = self.create_request()
        view = self.view_class.as_view()

        response = view(request, pk=discussion.pk)
        self.assertEqual(response.status_code, 200)
        comments = [older_comment, newer_comment]
        self.assertCountEqual(response.context_data['comments'], comments)
        self.assertEqual(response.context_data['discussion'], discussion)

    def test_post(self):
        discussion = factories.DiscussionFactory.create()
        user = self.user_factory.create()
        data = {'body': 'I am a comment!'}

        # Hit the view, passing in the necessaries.
        request = self.create_request('post', user=user, data=data)
        view = self.view_class.as_view()
        view(request, pk=discussion.pk)

        # Assert that one comment was created with the appropriate properties.
        created_comment = models.TextComment.objects.get(discussion=discussion)
        self.assertEqual(created_comment.body, data['body'])
        self.assertEqual(created_comment.discussion, discussion)
        self.assertEqual(created_comment.user, user)


class TestDiscussionCreate(RequestTestCase):
    view_class = discussions.DiscussionCreate

    def test_get(self):
        group = factories.GroupFactory.create()

        request = self.create_request()
        view = self.view_class.as_view()

        response = view(request, pk=group.pk)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.context_data['group'], group)

    def test_post(self):
        group = factories.GroupFactory.create()
        user = self.user_factory.create()
        data = {
            'creator': user,
            'group': group,
            'name': 'Super Sensible Discussion',
            'comment': 'Super sensible comment.',
        }

        self.assertFalse(models.Discussion.objects.exists())

        request = self.create_request('post', user=user, data=data)
        view = self.view_class.as_view()

        method_path = 'groups.views.discussions.DiscussionCreate.email_subscribers'
        with mock.patch(method_path) as email_subscribers:
            response = view(request, pk=group.pk)

        discussion = models.Discussion.objects.get()  # will explode if it doesn't exist
        self.assertEqual(discussion.creator, user)
        self.assertEqual(discussion.subscribers.get(), user)
        self.assertEqual(discussion.comments.get().body, 'Super sensible comment.')

        self.assertEqual(response.status_code, 302)
        expected = reverse('discussion-thread', kwargs={'pk': discussion.pk})
        self.assertEqual(response['Location'], expected)

        self.assertEqual(email_subscribers.call_count, 1)

    def test_email_subscribers(self):
        """
        Test notification emails for a new discussion.

        Assert that subscribers to a group are emailed correctly when a new discussion
        is posted there, apart from the discussion's creator.
        """
        group = factories.GroupFactory.create()
        subscriber = factories.UserFactory.create()
        creator = factories.UserFactory.create()
        group.subscribe(subscriber)
        group.subscribe(creator)

        new_discussion = factories.DiscussionFactory.create(creator=creator, group=group)
        first_comment = factories.TextCommentFactory.create(discussion=new_discussion)

        view = self.view_class()
        view.request = self.create_request(user=creator)

        reply_address = 'leeroy@jenkins.com'
        address_path = 'groups.views.discussions.get_reply_address'  # imports are weird
        with mock.patch(address_path, return_value=reply_address):
            view.email_subscribers(new_discussion)

        self.assertEqual(len(mail.outbox), 1)
        email = mail.outbox[0]
        self.assertEqual(email.subject, 'New discussion in {}'.format(group.name))
        self.assertEqual(email.to, [subscriber.email])
        self.assertEqual(email.reply_to, [reply_address])
        self.assertIn('A new discussion on "{}"'.format(new_discussion.name), email.body)
        self.assertIn(subscriber.get_full_name(), email.body)
        self.assertIn(first_comment.body, email.body)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('groups', '0012_filecomment'),
    ]

    operations = [
        migrations.AlterField(
            model_name='basecomment',
            name='polymorphic_ctype',
            field=models.ForeignKey(to='contenttypes.ContentType', null=True, related_name='polymorphic_groups.basecomment_set+', editable=False),
        ),
        migrations.AlterField(
            model_name='filecomment',
            name='file',
            field=models.FileField(upload_to='groups/file_comments'),
        ),
    ]



from django.contrib import admin

from .models import Discussion, Group


@admin.register(Group)
class GroupAdmin(admin.ModelAdmin):
    filter_horizontal = ('moderators', 'members_if_private')
    exclude = ('watchers',)

    class Meta:
        model = Group


@admin.register(Discussion)
class DiscussionAdmin(admin.ModelAdmin):
    exclude = ('subscribers', 'ignorers')

    class Meta:
        model = Discussion



from crispy_forms.bootstrap import FormActions, StrictButton
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout
from django import forms
from django.core.urlresolvers import reverse_lazy

from . import models


class BaseAddCommentForm(forms.ModelForm):
    """A base class for forms that create models inheriting from BaseComment."""
    def __init__(self, *args, **kwargs):
        super(BaseAddCommentForm, self).__init__(*args, **kwargs)
        self.helper = self.build_helper()

    def build_helper(self):
        """An overridable method that creates a crispy_forms layout helper."""
        helper = FormHelper()
        helper.form_class = 'form-horizontal'
        helper.label_class = 'col-lg-2'
        helper.field_class = 'col-lg-8'
        helper.layout = Layout(
            'body',
            FormActions(
                StrictButton('Post comment', type='submit'),
            ),
        )
        return helper

    class Meta:
        fields = ('body',)


class AddTextComment(BaseAddCommentForm):
    """A form that posts TextComments."""
    class Meta(BaseAddCommentForm.Meta):
        model = models.TextComment


class AddFileComment(BaseAddCommentForm):
    """A form that uploads FileComments."""
    def build_helper(self):
        helper = super(AddFileComment, self).build_helper()
        helper.layout = Layout(
            'file',
            FormActions(
                StrictButton('Upload this file', type='submit'),
            ),
        )
        return helper

    class Meta:
        fields = ('file',)
        model = models.FileComment


class DiscussionCreate(forms.Form):
    comment = forms.CharField(widget=forms.Textarea)
    name = forms.CharField(max_length=255)
    helper = FormHelper()
    helper.form_class = 'form-horizontal'
    helper.label_class = 'col-lg-2'
    helper.field_class = 'col-lg-8'
    helper.layout = Layout(
        'name',
        'comment',
        FormActions(
            StrictButton('Create Discussion', type='submit'),
        ),
    )


class SubscribeForm(forms.Form):
    subscribe = forms.BooleanField(widget=forms.HiddenInput(), required=False)

    class Meta:
        fields = ('subscribe',)

    def __init__(self, user, instance, url_name, *args, **kwargs):
        """
        Build the layout to reflect the action the form will take.

        Accepts (and requires) a user and an instance being subscribed to
        as keyword arguments.
        """
        to_subscribe = not instance.is_subscribed(user)
        initial_values = kwargs.setdefault('initial', {})
        initial_values['subscribe'] = to_subscribe

        super(SubscribeForm, self).__init__(*args, **kwargs)

        button_text = 'Subscribe' if to_subscribe else 'Unsubscribe'

        self.helper = FormHelper()
        self.helper.form_class = 'form-horizontal'
        self.helper.layout = Layout(
            FormActions(
                StrictButton(button_text, type='submit'),
            ),
        )
        self.helper.form_action = reverse_lazy(url_name, kwargs={'pk': instance.pk})



from setuptools import find_packages, setup


setup(
    version='3.1.1',
    name='incuna-groups',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'django_crispy_forms>=1.4.0,<2',
        'django-polymorphic>=0.7.2,<1',
    ],
    description='Generic group/forum framework.',
    author='Incuna Ltd',
    author_email='admin@incuna.com',
    url='https://github.com/incuna/incuna-groups',
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0001_initial'),
        ('groups', '0008_comment_state'),
    ]

    operations = [
        migrations.AddField(
            model_name='comment',
            name='polymorphic_ctype',
            field=models.ForeignKey(to='contenttypes.ContentType', null=True, related_name='polymorphic_groups.comment_set', editable=False),
            preserve_default=True,
        ),
    ]



from bs4 import BeautifulSoup
from crispy_forms.utils import render_crispy_form
from incuna_test_utils.compat import Python2AssertMixin
from incuna_test_utils.factories.images import uploadable_file

from . import factories
from .utils import RequestTestCase
from .. import forms, models


def has_submit(form):
    rendered_form = render_crispy_form(form)
    parser = BeautifulSoup(rendered_form, 'html.parser')
    submits = parser.findAll('input', {'type': 'submit'})
    return any(submits)


def get_button(form):
    rendered_form = render_crispy_form(form)
    return BeautifulSoup(rendered_form, 'html.parser').findAll('button')[0]


class TestAddTextComment(Python2AssertMixin, RequestTestCase):
    form = forms.AddTextComment
    model = models.TextComment

    def test_form_fields(self):
        expected = ['body']
        fields = self.form.base_fields.keys()
        self.assertCountEqual(fields, expected)

    def test_form_valid(self):
        data = {'body': 'I am a comment'}
        form = self.form(data=data)
        self.assertTrue(form.is_valid(), msg=form.errors)

    def test_form_not_valid(self):
        form = self.form(data={})
        self.assertFalse(form.is_valid())
        self.assertIn('body', form.errors)

    def test_submit_not_input(self):
        """The form does not have a submit <input>."""
        form = self.form()
        self.assertFalse(has_submit(form))

    def test_submit_button(self):
        """
        The form has a submit <button>.

        This allows for more flexibility in styling.
        """
        button = get_button(self.form())
        self.assertEqual(button.get('type'), 'submit')


class TestAddFileComment(Python2AssertMixin, RequestTestCase):
    form = forms.AddFileComment
    model = models.FileComment

    def test_form_fields(self):
        expected = ['file']
        fields = self.form.base_fields.keys()
        self.assertCountEqual(fields, expected)

    def test_form_valid(self):
        file_data = {'file': uploadable_file()}
        form = self.form(files=file_data)
        self.assertTrue(form.is_valid(), msg=form.errors)

    def test_form_not_valid(self):
        form = self.form(data={})
        self.assertFalse(form.is_valid())
        self.assertIn('file', form.errors)

    def test_submit_not_input(self):
        """The form does not have a submit <input>."""
        form = self.form()
        self.assertFalse(has_submit(form))

    def test_submit_button(self):
        """
        The form has a submit <button>.

        This allows for more flexibility in styling.
        """
        button = get_button(self.form())
        self.assertEqual(button.get('type'), 'submit')


class TestDiscussionCreateForm(Python2AssertMixin, RequestTestCase):
    form = forms.DiscussionCreate

    def test_form_fields(self):
        expected = ['name', 'comment']
        fields = self.form.base_fields.keys()
        self.assertCountEqual(fields, expected)

    def test_form_valid(self):
        data = {
            'name': "What's in a name?",
            'comment': 'First!!!!!!',
        }

        form = self.form(data=data)
        self.assertTrue(form.is_valid(), msg=form.errors)

    def test_form_not_valid(self):
        form = self.form(data={})
        self.assertFalse(form.is_valid())
        self.assertIn('name', form.errors)

    def test_submit_not_input(self):
        """The form does not have a submit <input>."""
        form = self.form()
        self.assertFalse(has_submit(form))

    def test_submit_button(self):
        """
        The form has a submit <button>.

        This allows for more flexibility in styling.
        """
        button = get_button(self.form())
        self.assertEqual(button.get('type'), 'submit')


class TestSubscribeForm(Python2AssertMixin, RequestTestCase):
    form = forms.SubscribeForm

    def setUp(self):
        self.discussion = factories.DiscussionFactory.create()
        self.user = self.discussion.creator

    def get_form(self, **kwargs):
        return self.form(
            user=self.user,
            instance=self.discussion,
            url_name='discussion-subscribe',
            **kwargs
        )

    def test_form_valid(self):
        form = self.get_form(data={})
        self.assertTrue(form.is_valid(), msg=form.errors)

    def test_initial_subscribe(self):
        form = self.get_form()
        self.assertTrue(form.initial['subscribe'])

    def test_initial_unsubscribe(self):
        self.discussion.subscribers.add(self.user)
        form = self.get_form()
        self.assertFalse(form.initial['subscribe'])

    def test_submit_not_input(self):
        """The form does not have a submit <input>."""
        form = self.get_form()
        self.assertFalse(has_submit(form))

    def test_submit_button(self):
        """
        The form has a submit <button>.

        This allows for more flexibility in styling.
        """
        button = get_button(self.get_form())
        self.assertEqual(button.get('type'), 'submit')

    def test_button_text_subscribe(self):
        """The button says 'Subscribe' when the user is not subscribed."""
        button = get_button(self.get_form())
        self.assertEqual(button.string, 'Subscribe')

    def test_button_text_unsubscribe(self):
        """The button says 'Unsubscribe' when the user is subscribed."""
        self.discussion.subscribers.add(self.user)
        button = get_button(self.get_form())
        self.assertEqual(button.string, 'Unsubscribe')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.utils.timezone
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('groups', '0009_comment_polymorphic_ctype'),
    ]

    operations = [
        migrations.CreateModel(
            name='BaseComment',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, auto_created=True, verbose_name='ID')),
                ('date_created', models.DateTimeField(default=django.utils.timezone.now)),
                ('state', models.CharField(max_length=255, choices=[('ok', 'OK'), ('deleted', 'Deleted')], default='ok')),
                ('discussion', models.ForeignKey(to='groups.Discussion', related_name='comments')),
                ('polymorphic_ctype', models.ForeignKey(null=True, editable=False, to='contenttypes.ContentType', related_name='polymorphic_groups.basecomment_set')),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL, related_name='comments')),
            ],
            options={
                'ordering': ('date_created',),
            },
            bases=(models.Model,),
        ),
        migrations.RemoveField(
            model_name='comment',
            name='discussion',
        ),
        migrations.RemoveField(
            model_name='comment',
            name='polymorphic_ctype',
        ),
        migrations.RemoveField(
            model_name='comment',
            name='user',
        ),
        migrations.DeleteModel(
            name='Comment',
        ),
    ]
